@@ -117,6 +117,16 @@ class PhygnnModel(ModelBase):
         """
         return self.model.history
 
+    @property
+    def version_record(self):
+        """A record of important versions that this model was built with.
+
+        Returns
+        -------
+        dict
+        """
+        return self.model.version_record
+
     def train_model(self, features, labels, p, n_batch=16, n_epoch=10,
                     shuffle=True, validation_split=0.2, run_preflight=True,
                     return_diagnostics=False, p_kwargs=None,
@@ -153,13 +163,13 @@ class PhygnnModel(ModelBase):
         validation_split : float
             Fraction of features and labels to use for validation.
         p_kwargs : None | dict
-            Optional kwargs for the physical loss function self._p_fun.
+            Optional kwargs for the physical loss function p_fun.
         run_preflight : bool
             Flag to run preflight checks.
         return_diagnostics : bool
             Flag to return training diagnostics dictionary.
         parse_kwargs : dict
-            kwargs for cls._parse_features
+            kwargs for cls.parse_features
         norm_labels : bool, optional
             Flag to normalize label, by default True
 
@@ -171,7 +181,7 @@ class PhygnnModel(ModelBase):
         if parse_kwargs is None:
             parse_kwargs = {}
 
-        x = self._parse_features(features, **parse_kwargs)
+        x = self.parse_features(features, **parse_kwargs)
         y = self._parse_labels(labels)
 
         diagnostics = self.model.fit(x, y, p,
@@ -296,17 +306,20 @@ class PhygnnModel(ModelBase):
                  {'dropout': 0.01},
                  {'class': 'Flatten'},
                  ]
-        input_layer : None | dict
+        input_layer : None | bool | dict
             Input layer. specification. Can be a dictionary similar to
             hidden_layers specifying a dense / conv / lstm layer.  Will
             default to a keras InputLayer with input shape = n_features.
-        output_layer : None | list | dict
+            Can be False if the input layer will be included in the
+            hidden_layers input.
+        output_layer : None | bool | list | dict
             Output layer specification. Can be a list/dict similar to
             hidden_layers input specifying a dense layer with activation.
             For example, for a classfication problem with a single output,
             output_layer should be [{'units': 1}, {'activation': 'sigmoid'}].
             This defaults to a single dense layer with no activation
-            (best for regression problems).
+            (best for regression problems).  Can be False if the output layer
+            will be included in the hidden_layers input.
         layers_obj : None | phygnn.utilities.tf_layers.Layers
             Optional initialized Layers object to set as the model layers
             including pre-set weights. This option will override the
@@ -471,17 +484,20 @@ class PhygnnModel(ModelBase):
                  {'dropout': 0.01},
                  {'class': 'Flatten'},
                  ]
-        input_layer : None | dict
+        input_layer : None | bool | dict
             Input layer. specification. Can be a dictionary similar to
             hidden_layers specifying a dense / conv / lstm layer.  Will
             default to a keras InputLayer with input shape = n_features.
-        output_layer : None | list | dict
+            Can be False if the input layer will be included in the
+            hidden_layers input.
+        output_layer : None } bool | list | dict
             Output layer specification. Can be a list/dict similar to
             hidden_layers input specifying a dense layer with activation.
             For example, for a classfication problem with a single output,
             output_layer should be [{'units': 1}, {'activation': 'sigmoid'}].
             This defaults to a single dense layer with no activation
-            (best for regression problems).
+            (best for regression problems).  Can be False if the output layer
+            will be included in the hidden_layers input.
         layers_obj : None | phygnn.utilities.tf_layers.Layers
             Optional initialized Layers object to set as the model layers
             including pre-set weights. This option will override the
@@ -536,9 +552,9 @@ class PhygnnModel(ModelBase):
             Flag to return training diagnostics dictionary.
             Fraction of features and labels to use for validation.
         p_kwargs : None | dict
-            Optional kwargs for the physical loss function self._p_fun.
+            Optional kwargs for the physical loss function p_fun.
         parse_kwargs : dict
-            kwargs for cls._parse_features
+            kwargs for cls.parse_features
         norm_labels : bool, optional
             Flag to normalize label, by default True
         save_path : str, optional
